<?php

/**
 * Name: IFTTT Receiver
 * Description: Receives a post from https://ifttt.com/ and distributes it.
 * Version: 0.1
 * Author: Michael Vogel <https://pirati.ca/profile/heluecht>
 */
use Friendica\App;
use Friendica\Content\PageInfo;
use Friendica\Core\Hook;
use Friendica\Core\Logger;
use Friendica\Core\Protocol;
use Friendica\Database\DBA;
use Friendica\DI;
use Friendica\Model\Item;
use Friendica\Model\Post;
use Friendica\Util\Strings;

function ifttt_install()
{
	Hook::register('connector_settings', 'addon/ifttt/ifttt.php', 'ifttt_settings');
	Hook::register('connector_settings_post', 'addon/ifttt/ifttt.php', 'ifttt_settings_post');
}

function ifttt_module()
{

}

function ifttt_content()
{

}

function ifttt_settings(App $a, &$s)
{
	if (!local_user()) {
		return;
	}

	$key = DI::pConfig()->get(local_user(), 'ifttt', 'key');

	if (!$key) {
		$key = Strings::getRandomHex(20);
		DI::pConfig()->set(local_user(), 'ifttt', 'key', $key);
	}

	$s .= '<span id="settings_ifttt_inflated" class="settings-block fakelink" style="display: block;" onclick="openClose(\'settings_ifttt_expanded\'); openClose(\'settings_ifttt_inflated\');">';
	$s .= '<img class="connector" src="addon/ifttt/ifttt.png" /><h3 class="connector">' . DI::l10n()->t('IFTTT Mirror') . '</h3>';
	$s .= '</span>';
	$s .= '<div id="settings_ifttt_expanded" class="settings-block" style="display: none;">';
	$s .= '<span class="fakelink" onclick="openClose(\'settings_ifttt_expanded\'); openClose(\'settings_ifttt_inflated\');">';
	$s .= '<img class="connector" src="addon/ifttt/ifttt.png" /><h3 class="connector">' . DI::l10n()->t('IFTTT Mirror') . '</h3>';
	$s .= '</span>';

	$s .= '<div id="ifttt-configuration-wrapper">';
	$s .= '<p>' . DI::l10n()->t('Create an account at <a href="http://www.ifttt.com">IFTTT</a>. Create three Facebook recipes that are connected with <a href="https://ifttt.com/maker">Maker</a> (In the form "if Facebook then Maker") with the following parameters:') . '</p>';
	$s .= '<h4>URL</h4>';
	$s .= '<p>' . DI::baseUrl()->get() . '/ifttt/' . $a->getLoggedInUserNickname() . '</p>';
	$s .= '<h4>Method</h4>';
	$s .= '<p>POST</p>';
	$s .= '<h4>Content Type</h4>';
	$s .= '<p>application/x-www-form-urlencoded</p>';
	$s .= '<h4>' . DI::l10n()->t('Body for "new status message"') . '</h4>';
	$s .= '<p><code>' . htmlentities('key=' . $key . '&type=status&msg=<<<{{Message}}>>>&date=<<<{{UpdatedAt}}>>>&url=<<<{{PageUrl}}>>>') . '</code></p>';
	$s .= '<h4>' . DI::l10n()->t('Body for "new photo upload"') . '</h4>';
	$s .= '<p><code>' . htmlentities('key=' . $key . '&type=photo&link=<<<{{Link}}>>>&image=<<<{{ImageSource}}>>>&msg=<<<{{Caption}}>>>&date=<<<{{CreatedAt}}>>>&url=<<<{{PageUrl}}>>>') . '</code></p>';
	$s .= '<h4>' . DI::l10n()->t('Body for "new link post"') . '</h4>';
	$s .= '<p><code>' . htmlentities('key=' . $key . '&type=link&link=<<<{{Link}}>>>&title=<<<{{Title}}>>>&msg=<<<{{Message}}>>>&description=<<<{{Description}}>>>&date=<<<{{CreatedAt}}>>>&url=<<<{{PageUrl}}>>>') . '</code></p>';
	$s .= '</div><div class="clear"></div>';

	$s .= '<div id="ifttt-rekey-wrapper">';
	$s .= '<label id="ifttt-rekey-label" for="ifttt-checkbox">' . DI::l10n()->t('Generate new key') . '</label>';
	$s .= '<input id="ifttt-checkbox" type="checkbox" name="ifttt-rekey" value="1" />';
	$s .= '</div><div class="clear"></div>';

	$s .= '<div class="settings-submit-wrapper" ><input type="submit" name="ifttt-submit" class="settings-submit" value="' . DI::l10n()->t('Save Settings') . '" /></div>';
	$s .= '</div>';
}

function ifttt_settings_post()
{
	if (!empty($_POST['ifttt-submit']) && isset($_POST['ifttt-rekey'])) {
		DI::pConfig()->delete(local_user(), 'ifttt', 'key');
	}
}

function ifttt_post(App $a)
{
	if (DI::args()->getArgc() != 2) {
		return;
	}

	$nickname = DI::args()->getArgv()[1];

	$user = DBA::selectFirst('user', ['uid'], ['nickname' => $nickname]);
	if (!DBA::isResult($user)) {
		Logger::info('User ' . $nickname . ' not found.');
		return;
	}

	$uid = $user['uid'];

	Logger::info('Received a post for user ' . $uid . ' from ifttt ' . print_r($_REQUEST, true));

	if (!isset($_REQUEST['key'])) {
		Logger::notice('No key found.');
		return;
	}

	$key = $_REQUEST['key'];

	// Check the key
	if ($key != DI::pConfig()->get($uid, 'ifttt', 'key')) {
		Logger::info('Invalid key for user ' . $uid);
		return;
	}

	$item = [];

	if (isset($_REQUEST['type'])) {
		$item['type'] = $_REQUEST['type'];
	}

	if (!in_array($item['type'], ['status', 'link', 'photo'])) {
		Logger::info('Unknown item type ' . $item['type']);
		return;
	}

	if (isset($_REQUEST['link'])) {
		$item['link'] = trim($_REQUEST['link']);
	}
	if (isset($_REQUEST['image'])) {
		$item['image'] = trim($_REQUEST['image']);
	}
	if (isset($_REQUEST['title'])) {
		$item['title'] = trim($_REQUEST['title']);
	}
	if (isset($_REQUEST['msg'])) {
		$item['msg'] = trim($_REQUEST['msg']);
	}
	if (isset($_REQUEST['description'])) {
		$item['description'] = trim($_REQUEST['description']);
	}
	if (isset($_REQUEST['date'])) {
		$item['date'] = date('c', strtotime($date = str_replace(' at ', ', ', $_REQUEST['date'])));
	}
	if (isset($_REQUEST['url'])) {
		$item['url'] = trim($_REQUEST['url']);
	}

	if ((substr($item['msg'], 0, 3) == '<<<') && (substr($item['msg'], -3, 3) == '>>>')) {
		$item['msg'] = substr($item['msg'], 3, -3);
	}

	ifttt_message($uid, $item);
}

function ifttt_message($uid, $item)
{
	$a = DI::app();

	$post = [];
	$post['uid'] = $uid;
	$post['app'] = 'IFTTT';
	$post['title'] = '';
	$post['body'] = $item['msg'];
	//$post['date'] = $item['date'];
	//$post['uri'] = $item['url'];

	if ($item['type'] == 'link') {
		$link = $item['link'];
		$data = PageInfo::queryUrl($item['link']);

		if (isset($item['title']) && (trim($item['title']) != '')) {
			$data['title'] = $item['title'];
		}

		if (isset($item['description']) && (trim($item['description']) != '')) {
			$data['text'] = $item['description'];
		}

		$post['body'] .= "\n" . PageInfo::getFooterFromData($data);
	} elseif (($item['type'] == 'photo') && ($item['image'] != '')) {
		$link = $item['image'];
		$post['body'] .= "\n\n[img]" . $item['image'] . "[/img]\n";
	} elseif (!empty($item['url'])) {
		$link = $item['url'];
	} else {
		$link = hash('ripemd128', $item['msg']);
	}

	Post\Delayed::add($link, $post, PRIORITY_MEDIUM, Post\Delayed::UNPREPARED);
}
